(function () {
    'use strict';

    angular
        .module('cml-adherence-app')
        .factory('cmlMedicationsService', cmlMedicationsService);

    function cmlMedicationsService($rootScope, $log, $filter, cmlLocalStorageService, cmlLocalNotificationsService, _, $translate) {
        $log.debug('Hello cmlMedicationsService');

        var STORAGE_KEY = 'cml.medications';
        var self = this;

        var _id = 0;
        var _medications = [];

        function getNewId() {
            return (++_id);
        }

        function serialize() {
            var _storageData = {
                lastIndex: _id,
                medications: _medications
            };
            cmlLocalStorageService.setObject(STORAGE_KEY, _storageData);

            syncWithNotifications();
        }

        function deserialize() {
            var _storage = cmlLocalStorageService.getObject(STORAGE_KEY);

            if (angular.isDefined(_storage)) {
                _id = _storage.lastIndex;
                _medications = _storage.medications;

                _.forEach(_medications, function (medication) {
                    medication.doses = _.forEach(medication.doses, function (dose) {
                        dose.time = cmlLocalStorageService.strToDate(dose.time);
                    });
                    medication.createdAt = cmlLocalStorageService.strToDate(medication.createdAt);
                });
            }

            syncWithNotifications();
        }

        function fixedDoseTime(dose, interval) {
            var now = new Date();
            var time = new Date(dose.time);

            // is time in the past?
            if (now.getTime() > time.getTime()) {
                if (interval === 'day') {
                    time.setFullYear(
                        now.getFullYear(),
                        now.getMonth(),
                        now.getDate() + 1
                    );
                } else {
                    var delta = (7 - (now.getDay() - dose['weekDay']));

                    time.setFullYear(
                        now.getFullYear(),
                        now.getMonth(),
                        now.getDate() + delta
                    );
                }
            }

            $log.debug(dose.time, time);

            return time;
        }

        function syncWithNotifications() {
            cmlLocalNotificationsService.getAllNotifications().then(function (ids) {
                var notificationIds = ids;
                var newNotifications = [];
                var updateNotifications = [];

                _.forEach(_medications, function (medication) {
                    if (medication.reminder) {
                        _.forEach(medication.doses, function (dose) {
                            var indexOfNotification = notificationIds.indexOf(dose.id);
                            var reminderText = angular.isDefined(medication['reminderText']) ? medication['reminderText'].trim() : '';
                            var reminderAltText = self.getDefaultReminderText(medication);

                            if (_.isEmpty(reminderText)) {
                                reminderText = reminderAltText;
                            }

                            var notification = cmlLocalNotificationsService.createNotification(
                                dose.id,
                                reminderText,
                                reminderAltText,
                                fixedDoseTime(dose, medication['interval']),
                                medication['interval']
                            );

                            if (indexOfNotification === -1) {
                                newNotifications.push(notification);
                            } else {
                                notificationIds.splice(indexOfNotification, 1);
                                updateNotifications.push(notification);
                            }
                        });
                    }
                });

                // schedule new notifications
                if (!_.isEmpty(newNotifications)) {
                    cmlLocalNotificationsService.addNotifications(newNotifications);
                }

                // update notifications
                if (!_.isEmpty(updateNotifications)) {
                    cmlLocalNotificationsService.updateNotifications(updateNotifications);
                }

                // cancel old notifications
                if (!_.isEmpty(notificationIds)) {
                    cmlLocalNotificationsService.removeNotifications(notificationIds);
                }
            });
        }

        self.init = function () {
            deserialize();
            $rootScope.$broadcast('cmlMedicationsInit');
        };

        self.getDefaultReminderText = function (medication) {
            return medication.title + ' ' + medication.amount + ' ' + $translate.instant($filter('cmlUnit')(medication.unit));
        };

        self.getMedicationByDoseId = function (doseId) {
            var _medication = undefined;

            _.forEach(_medications, function (medication) {
                var dose = _.find(medication.doses, 'id', parseInt(doseId));
                if (angular.isDefined(dose)) {
                    _medication = medication;
                    return false;
                }
            });

            return _medication;
        };

        self.getAll = function () {
            return _medications;
        };

        self.add = function (medication) {
            // add IDs
            medication.id = getNewId();

            _.forEach(medication.doses, function (dose) {
                dose.id = getNewId();
            });

            medication.createdAt = new Date();

            // save
            _medications.push(medication);
            serialize();
        };

        self.get = function (id) {
            var medication = _.find(_medications, 'id', parseInt(id));
            return _.cloneDeep(medication);
        };

        self.remove = function (id) {
            var index = _.findIndex(_medications, 'id', parseInt(id));

            if (index > -1) {
                _medications.splice(index, 1);
                serialize();
            }
        };

        self.update = function (updatedMedication) {
            // set ID for new doses
            _.forEach(updatedMedication.doses, function (dose) {
                if (angular.isUndefined(dose.id)) {
                    dose.id = getNewId();
                }
            });

            // update medication
            _.forEach(_medications, function (medication) {
                if (medication.id == updatedMedication.id) {
                    _.assign(medication, updatedMedication);
                    return false;
                }
            });

            serialize();
        };

        return self;
    }
})();
